"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.ReplaceOperation = exports.ReplaceStrategy = void 0;
var range_1 = require("../range");
var serializable_1 = require("../serializable");
var AbstractOperation_1 = require("./AbstractOperation");
var fillConstantsInString_1 = require("./utils/fillConstantsInString");
var ReplaceStrategy;
(function (ReplaceStrategy) {
    ReplaceStrategy["ALL"] = "all";
    ReplaceStrategy["ONCE"] = "once";
})(ReplaceStrategy = exports.ReplaceStrategy || (exports.ReplaceStrategy = {}));
function applyReplaceLinesRangeOnce(sourceText, searchValue, replaceValue) {
    var lines = sourceText.split('\n');
    var startLineIndex = lines.findIndex(function (line) {
        return line.includes(searchValue.startToken);
    });
    var endLineIndex = lines.findIndex(function (line, lineIndex) {
        return lineIndex > startLineIndex && line.includes(searchValue.endToken);
    });
    if (startLineIndex === -1 || endLineIndex === -1) {
        return null;
    }
    var range = new range_1.TextLinesRange(startLineIndex, endLineIndex);
    return new ReplaceOperation(range, replaceValue, ReplaceStrategy.ONCE).apply(sourceText);
}
function applyReplaceTextRange(sourceText, searchValue, replaceValue) {
    var startIndex = sourceText.indexOf(searchValue.startToken);
    var endIndex = sourceText.indexOf(searchValue.endToken, startIndex);
    if (startIndex === -1 || endIndex === -1) {
        return null;
    }
    return new ReplaceOperation(new range_1.TextRange(startIndex, endIndex + searchValue.endToken.length - 1), replaceValue, ReplaceStrategy.ONCE).apply(sourceText);
}
var ReplaceOperation = /** @class */ (function (_super) {
    __extends(ReplaceOperation, _super);
    function ReplaceOperation(searchValue, replaceValue, repeatCount) {
        if (repeatCount === void 0) { repeatCount = ReplaceStrategy.ONCE; }
        var _this = _super.call(this) || this;
        _this.searchValue = searchValue;
        _this.replaceValue = replaceValue;
        _this.repeatCount = repeatCount;
        _this.SERIALIZABLE_CLASS = ReplaceOperation.name;
        return _this;
    }
    ReplaceOperation.prototype.toSerializable = function () {
        var searchValue = typeof this.searchValue === 'string'
            ? this.searchValue
            : this.searchValue.toSerializable();
        return {
            SERIALIZABLE_CLASS: this.SERIALIZABLE_CLASS,
            searchValue: searchValue,
            replaceValue: this.replaceValue,
            repeatCount: this.repeatCount
        };
    };
    ReplaceOperation.fromSerializable = function (serializable, constants) {
        var replaceValue = constants
            ? (0, fillConstantsInString_1.fillConstantsInString)(serializable.replaceValue, constants)
            : serializable.replaceValue;
        var searchValue = typeof serializable.searchValue === 'string'
            ? serializable.searchValue
            : (0, serializable_1.rangeFromSerializable)(serializable.searchValue);
        return new ReplaceOperation(searchValue, replaceValue, serializable.repeatCount);
    };
    ReplaceOperation.prototype.apply = function (sourceText) {
        var _this = this;
        var searchValue = this.searchValue;
        if (typeof searchValue === 'string') {
            if (typeof this.repeatCount === 'number') {
                var newSource = sourceText;
                for (var i = this.repeatCount; i > 0; i--) {
                    newSource = sourceText.replace(searchValue, this.replaceValue);
                }
                return newSource;
            }
            switch (this.repeatCount) {
                case ReplaceStrategy.ONCE:
                    return sourceText.replace(searchValue, this.replaceValue);
                case ReplaceStrategy.ALL:
                    return sourceText.replace(new RegExp(searchValue, 'g'), this.replaceValue);
            }
        }
        if (searchValue instanceof range_1.TextRange) {
            if (searchValue.startIndex >= sourceText.length ||
                searchValue.endIndex > sourceText.length) {
                return sourceText;
            }
            if (searchValue.startIndex >= searchValue.endIndex) {
                return sourceText;
            }
            return (sourceText.slice(0, searchValue.startIndex - 1) +
                this.replaceValue +
                sourceText.slice(searchValue.endIndex, sourceText.length));
        }
        if (searchValue instanceof range_1.TextLinesRange) {
            if (searchValue.startIndex >= sourceText.length ||
                searchValue.endIndex > sourceText.length) {
                return sourceText;
            }
            if (searchValue.startIndex >= searchValue.endIndex) {
                return sourceText;
            }
            var lines_1 = sourceText.split('\n');
            var valueHasReplaced_1 = false;
            return lines_1.reduce(function (acc, line, lineIndex) {
                if (lineIndex >= searchValue.startIndex &&
                    lineIndex <= searchValue.endIndex) {
                    if (!valueHasReplaced_1) {
                        valueHasReplaced_1 = true;
                        return acc + _this.replaceValue;
                    }
                    return acc;
                }
                var isLastLine = lineIndex === lines_1.length - 1;
                var lineBreak = isLastLine ? '' : '\n';
                return acc + "".concat(line).concat(lineBreak);
            }, '');
        }
        if (searchValue instanceof range_1.TextSection ||
            searchValue instanceof range_1.TextLinesSection) {
            var doApplyOnce = searchValue instanceof range_1.TextSection
                ? applyReplaceTextRange
                : applyReplaceLinesRangeOnce;
            if (typeof this.repeatCount === 'number') {
                var newSourceText = sourceText;
                for (var i = this.repeatCount; i > 0; i--) {
                    var result = doApplyOnce(newSourceText, searchValue, this.replaceValue);
                    if (!result) {
                        break;
                    }
                    newSourceText = result;
                }
                return newSourceText;
            }
            switch (this.repeatCount) {
                case ReplaceStrategy.ONCE: {
                    var newSourceText = doApplyOnce(sourceText, searchValue, this.replaceValue);
                    if (!newSourceText) {
                        return sourceText;
                    }
                    return newSourceText;
                }
                case ReplaceStrategy.ALL: {
                    var newSourceText = sourceText, continueLoop = true;
                    while (continueLoop) {
                        var result = doApplyOnce(newSourceText, searchValue, this.replaceValue);
                        if (!result) {
                            continueLoop = false;
                            break;
                        }
                        newSourceText = result;
                    }
                    return newSourceText;
                }
            }
        }
        return sourceText;
    };
    return ReplaceOperation;
}(AbstractOperation_1.AbstractOperation));
exports.ReplaceOperation = ReplaceOperation;
//# sourceMappingURL=ReplaceOperation.js.map